package ij.example.dvdshop;

import ij.util.*;

import java.util.*;

public class DVDShop implements XMLFragment{       
  protected List<DVD> dvds=new ArrayList<DVD>();
  public List<DVD> getDVDs(){ return dvds; }
  public void add(DVD dvd){ dvds.add(dvd); }

  public DVDShop(){ }  
  public DVDShop(DVD dvds[]){ this.dvds.addAll(Arrays.asList(dvds)); }
  public int getDVDCount(){ return dvds.size(); }

  public int getBorrowedDVDCount(){ int n=0;
    for(DVD dvd :getDVDs()){
      if(dvd.isBorrowed()) ++n;
    }
    return n;
  }
  
  public double getTotalPrice(){ double s=0;
    for(DVD dvd :getDVDs()){
      if(dvd.isForSale()) s+=dvd.getPrice();
    }
    return s;
  }
  public double getDVDForSaleCount(){ int n=0;
    for(DVD dvd :getDVDs()){
      if(dvd.isForSale()) ++n;
    }
    return n;
  }
  
  public Collection<String> getActors(){ 
    Collection<String> actors=new TreeSet<String>(new NameComparator());
    for(DVD dvd: getDVDs()){ 
      actors.addAll( StringUtil.trim( 
          StringUtil.splitToList(dvd.getActors(),',') 
      ) );
    }
    return actors;
  }
  
    
  public String appendXMLHead(StringBuilder b, String ident){
    b.append(ident).append("<DVDShop>\n");
    return "DVDShop";
  }    
  public XMLFragment[] appendXMLBody(StringBuilder b, String ident){     
     return dvds.toArray(new XMLFragment[0]);
  }    
  
  public static void main(String args[]){
    class CodeComparator implements Comparator<DVD>{                                            
      public int compare(DVD a, DVD b){ if(a==b) return 0;
        return a.getCode().compareTo(b.getCode());
      }
    }  
    
    DVDShop shop=new DVDShop(); DVD dvd;
    shop.add(dvd=new DVD("1001", "Nights on Java", 2006, "Ben Boss",
               "Dev E. Lopez, T. Esther Works"));
    dvd.setBorrowed(true); dvd.setPrice(2000);
    shop.add(dvd=new DVD("1010", "Matrix",1999,"Watchowsky bros.",
               "Keanu Reeves, Laurence Fishburne, Carrie-Anne Moss" ));
    dvd.setBorrowed(true); 
    shop.add(dvd=new DVD("1011", "The Fifth Element",1997,"Luc Besson",
               "Bruce Willis, Gary Oldman, Ian Holm, Milla Jovovich" ));
    shop.add(dvd=new DVD("1012", "Hamlet",1990,"Franco Zeffirelli ",
               "Mel Gibson , Glenn Close, Helena Bonham Carter" ));
    dvd.setBorrowed(true); dvd.setPrice(1200);
    shop.add(dvd=new DVD("1013", "Ladyhawke",1985,"Richard Donner",
               "Matthew Broderick, Rutger Hauer, Michelle Pfeiffer" ));
    dvd.setPrice(1000);
    /*
    int borrowedDVDCount=shop.getBorrowedDVDCount();
    System.out.println("Klcsnztt DVD-k szma: "
        +borrowedDVDCount
        +" ( "+ (double)borrowedDVDCount/shop.getDVDCount()*100 +"% )"  
    );
    
    double totalPrice=shop.getTotalPrice();
    System.out.println("sszr: "+totalPrice );
    System.out.println("tlag:  "+totalPrice/shop.getDVDForSaleCount() ); */
    
    //System.out.println( XMLBuilder.toXML(shop) );
    //System.out.println(shop.getActors());
    
    SortedSet<DVD> set=new TreeSet<DVD>(new CodeComparator());
    set.addAll(shop.getDVDs());
    System.out.println(set);
  }
}
