package ij.example.simple;

import java.io.*;
import java.util.*;

public class DoubleFile extends AbstractList<Double> implements Closeable{

  protected RandomAccessFile rfile;
  public DoubleFile(File file) throws FileNotFoundException{
    rfile=new RandomAccessFile(file,"rw");
  }

  public void close() throws IOException{ rfile.close(); }

  protected IOException exception;
  public IOException getException(){ return exception; }
  public void setException(IOException exception){ 
    if(this.exception==null) this.exception=exception; 
  }
  public void throwExceptionIfExists() throws IOException{ 
     if(exception!=null){ 
       IOException e=exception; exception=null;
       throw e; 
     }
  }
  
  public void clear() {
    try{ rfile.setLength(0);
    }catch(IOException e){ setException(e);
    }
  }   
  public int size() {
    try{
      return (int)rfile.length()/8;
    }catch (IOException e){ setException(e);
      return 0;
    } 
  }
  public Double get(int index) {
    try{      
      rfile.seek(index*8);
      return rfile.readDouble();
    }catch(IOException e){ setException(e);     
      return null;
    }
  }
  public Double set(int index, Double v) {
    try{
      rfile.seek(index*8);
      rfile.writeDouble(v);
    }catch(IOException e){ setException(e);
    }
    return null;
  }
  public void add(int index, Double v) {
    if(index!=size()) throw new IndexOutOfBoundsException();
    try{
      rfile.seek(index*8);
      rfile.writeDouble(v);
    }catch(IOException e){ setException(e);
    }
  }

  public static void main(String[] args) throws IOException{
    DoubleFile df=new DoubleFile(new File("data.dat"));
 // example data
    df.clear();
    df.addAll(Arrays.asList(new Double[]{ 7.0, 3.0, 4.0, 2.0, 3.0 }));
 // load
    List<Double> list=new ArrayList(df);
    System.out.println(list);
 // sort   
    Collections.sort(list);
    System.out.println(list);
 // store
    df.clear(); df.addAll(list);
    System.out.println(df);    
 // close
    df.close(); df.throwExceptionIfExists();
  }
}
