package ij.util;

import java.util.*;

public abstract class AbstractKeyBasedMap<K,V> extends AbstractMap<K,V>{

  public class KeyBasedEntryCursor 
                     implements Iterator<Map.Entry<K,V>>, Map.Entry<K,V>{
    protected Iterator<K> keyIterator;
    protected K key;
    public K getKey(){ return key; }
    public V getValue(){ return get(key); }
    public V setValue(V value){ return put(key,value); }
    public KeyBasedEntryCursor(){
      this.keyIterator=keySet().iterator();  
    }
    public boolean hasNext(){ return keyIterator.hasNext(); }
    public Map.Entry<K,V> next(){ key=keyIterator.next(); return this; }
    public void remove(){ keyIterator.remove(); }
  }
  public class KeyBasedEntrySet extends AbstractSet<Map.Entry<K,V>>{
    public int size(){ return keySet().size(); } 
    public Iterator<Map.Entry<K,V>> iterator(){ 
      return new KeyBasedEntryCursor(); 
    }
  }

  public abstract Set<K> keySet();  
  public abstract V get(Object key);
  public abstract V put(K key, V value);

  protected Set<Map.Entry<K,V>> entrySet;
  public Set<Map.Entry<K,V>> entrySet(){ 
    if(entrySet==null) entrySet=new KeyBasedEntrySet();
    return entrySet; 
  }
  public AbstractKeyBasedMap(){ }
  public AbstractKeyBasedMap(Map<K,V> map){ putAll(map); }

  public Map<K,V> toMap(){ return this; }
}
