package ij.util;

import ij.example.dvdshop.DVD;

import java.beans.*;
import java.util.*;

public class BeanMap extends AbstractKeyBasedMap<String,Object>{
  protected Class theClass;
  protected Object object;

  public Object getBean(){ return object; }
  public boolean isEmpty(){ return object==null; }
  
  protected PropertyDescriptor descriptors[];
  protected Set<String> keys=new LinkedHashSet();
  public Set<String> keySet(){ return keys; }
  public BeanMap(Class theClass, Object object) throws IntrospectionException{ 
    this.theClass= (theClass!=null) ? theClass : object.getClass();
    this.object=object;
    descriptors=Introspector.getBeanInfo(this.theClass)
                              .getPropertyDescriptors();
    for(PropertyDescriptor descriptor: descriptors){
      keys.add(descriptor.getName());
    }   
  }
  public BeanMap(Object object) throws IntrospectionException{ 
    this(null,object); 
  }
  public BeanMap(Class theClass) throws IntrospectionException{ 
    this(theClass,null); 
  }

  public Object get(Object name){
    if(object==null) return null;    
    try{
      for(PropertyDescriptor descriptor: descriptors){
        if(name.equals(descriptor.getName())){
          return descriptor.getReadMethod().invoke(object);
        }
      }
    }catch(Exception e){ }
    return null; 
  }
  public Object put(String name, Object value){
    try{
      if(object==null){ object=theClass.newInstance(); }      
      for(PropertyDescriptor d: descriptors){
        if(name.equals(d.getName())){
          if(d.getPropertyType()!=String.class 
                                             && value instanceof String){
            PropertyEditor pe=PropertyEditorManager
                                        .findEditor(d.getPropertyType());
            if(pe!=null){
              try{
                pe.setAsText((String)value);
                return d.getWriteMethod().invoke(object,pe.getValue());
              }catch(Exception e){ }
            }
          }
          return d.getWriteMethod().invoke(object,value);
        }
      }
    }catch(Exception e){}
    return null; 
  }
  
  public static Map asMap(Object object){
    if(object instanceof Map) return (Map)object;
    if(object==null) return Collections.EMPTY_MAP;
    try{
      return new BeanMap(object);
    }catch(IntrospectionException e){
      throw new RuntimeException(e);
    }
  }  
  
  public static void main(String args[]) throws IntrospectionException{
    DVD dvd=new DVD();
    dvd.setCode("2121");
    dvd.setTitle("Nights on Java"); dvd.setReleaseDate(2006);    
    dvd.setDirector("Ben Boss");
    dvd.setActors("Dev E. Lopez, T. Esther Works");

    BeanMap b=new BeanMap(dvd);
    System.out.println( b.get("title") );
    
    BeanMap d=new BeanMap( DVD.class );
    d.put("title","Working title"); 
    System.out.println( ((DVD)d.getBean()).getTitle() );

    b.put("title","Nights with JavaBeans");
    System.out.println( b );
    
  }
}
