package ij.util;

import ij.example.dvdshop.DVD;
import ij.example.dvdshop.DVDShop;

import java.io.*;
import java.util.*;

public class CSVReader<T> implements Iterator<T>,Iterable<T>{
  public static int readValue(StringBuilder b, String ln, int i, 
                                                             String sep){
    if(i>=ln.length()) return -1;
    int p=i; b.setLength(0);
    if(ln.startsWith("\"",i)){ p=++i;
      for( ; (i=ln.indexOf("\"",i))!=-1; ++i){ 
        if(ln.startsWith("\"",i+1)){ 
          b.append(ln,p,i);
          p=++i; 
        }else{ break;         
        }
      }
      if(i==-1) i=ln.length();
      b.append(ln,p,i);
      if(ln.startsWith("\"",i)) ++i;
    }else{ 
      if( (i=ln.indexOf(sep,i))==-1 ){ i=ln.length(); }
      b.append(ln,p,i);
    }    
    if(ln.startsWith(sep,i)) ++i;
    return i;
  }
  public static Map readValues(Map map, Iterable keys, String ln, String sep){
    StringBuilder b=new StringBuilder();
    int index=0; 
    for(Object key: keys){
      if( (index=readValue(b,ln,index,sep))==-1 ) break;
      map.put(key,b.toString());
    }
    return map;
  }
  public static List readValues(List keys, String ln, String sep){
    StringBuilder b=new StringBuilder();
    int index=0; 
    for( ; ;){
      if((index=readValue(b,ln,index,sep))==-1 ) break;
      keys.add(b.toString());
    }
    return keys;
  }
  
  protected BufferedReader in;
  protected String ln;
  protected IOException exception;
  public void throwExceptionIfExists() throws IOException{
    IOException e=exception; exception=null;
    if(e!=null) throw e;
  }
  protected void readLine(){ ln=null; 
    try{ 
      if((ln=in.readLine())==null){
        in.close(); 
      }
    }catch(IOException e){ 
      if(exception==null) exception=e;
    }
  }  
  
  protected Iterable keys;  
  public CSVReader(Reader in, Iterable keys) { 
    this.in = (in instanceof BufferedReader) 
      ? (BufferedReader)in
      : new BufferedReader(in);
    readLine();
    if(keys==null && ln!=null){
      keys=readValues(new ArrayList(),ln,";");
      readLine();
    }
    this.keys=keys;    
  }
  public CSVReader(Reader in) { this(in,null); }
  public CSVReader(String fname, Iterable keys) throws FileNotFoundException{ 
    this(new FileReader(fname),null); 
  } 
  public CSVReader(String fname) throws FileNotFoundException{ this(fname,null); } 
  
  protected Map buffer=new LinkedHashMap();
  
  public boolean hasNext(){ return ln!=null; }
  public T next(){ 
    buffer.clear(); readValues(buffer,keys,ln,";"); 
    readLine(); 
    T object=new T(); object.putAll(buffer);
    return object;    
  }
  public void remove(){ throw new UnsupportedOperationException(); }  
  
  protected boolean iterable=true;
  public Iterator<T> iterator(){
    if(iterable){ iterable=false; return this; } 
    throw new IllegalStateException("Repeated call of iterator()");
  }  
  
  public static void main(String args[])throws IOException{
    DVDShop shop=new DVDShop();
    for(DVD dvd :new CSVReader<DVD>("movie.csv")){
      shop.add(dvd);
    }
    System.out.println(XMLBuilder.toXML(shop));
  }
}
