package ij.util;

import java.io.*;
import java.sql.*;
import java.util.*;

public class DB{
  protected Connection conn;
  protected void connect() throws ClassNotFoundException, SQLException{
    Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
    conn=DriverManager.getConnection(
      "jdbc:derby:forsale_db;create=true", "forsale", "passw"
    );
  }
  public DB() throws ClassNotFoundException, SQLException{ 
    connect();
  }
  public void close(){ 
    try{ conn.close(); }catch(SQLException e){}
  }
  
  public static PreparedStatement prepareStatement(Connection conn, String stmt, 
                                Map<? extends Object, ? extends Object> op) throws SQLException{
    StringBuilder b=null; List<Object> list=null; 
    int i=0;
    for(int j,k; (j=stmt.indexOf("${",i))!=-1 
              && (k=stmt.indexOf("}",j))!=-1; i=k+1){
      String key=stmt.substring(j+2,k); 
      if(b==null){ 
        b=new StringBuilder(stmt.length()); 
        list=new ArrayList<Object>(); 
      }
      b.append(stmt,i,j).append('?');
      list.add(op.get(key));
    }
    if(b!=null){
      if(i<stmt.length()) b.append(stmt,i,stmt.length());
      PreparedStatement statement=conn.prepareStatement(b.toString());
      i=0;
      for(Object o: list){ 
        statement.setObject(++i,o);
      }
      return statement;
    }else{
      return conn.prepareStatement(stmt);
    }
  }
  public ResultSetIterator select(String stmt, 
                            Map<? extends Object, ? extends Object> op) throws SQLException{
    if(!stmt.regionMatches(true,0,"SELECT",0,6)) stmt="SELECT "+stmt;
    return new ResultSetIterator(prepareStatement(conn,stmt,op));
  }
  public ResultSetIterator select(String stmt, Object... op) throws SQLException{
    return select(stmt, new ArrayMap(op));
  }
  
  public ResultSetIterator select(String stmt) throws SQLException{
    if(!stmt.regionMatches(true,0,"SELECT",0,6)) stmt="SELECT "+stmt;
    return new ResultSetIterator(conn.prepareStatement(stmt));
  }
  
  public int delete(String table, String key, Object id) throws SQLException{ 
    StringBuilder cmd=new StringBuilder()
      .append("DELETE FROM ").append(table)
      .append(" WHERE ").append(key).append("=?");
    PreparedStatement statement=null;
    try{ 
      statement=conn.prepareStatement(cmd.toString());
      statement.setObject(1,id);
      return statement.executeUpdate();
    }finally{
      try{ statement.close(); }catch(Exception e){}
    }
  }
  public int update(String table, String key, 
      Map<? extends Object, ? extends Object> op) throws SQLException{ 
    StringBuilder cmd=new StringBuilder()
      .append("UPDATE ").append(table).append(" SET ");
    String sep="";
    for(Map.Entry e: op.entrySet()){ String n=e.getKey().toString();
      if(n.equals(key)) continue;
      cmd.append(sep).append(n).append("=?");
      sep=","; 
    }
    cmd.append(" WHERE ").append(key).append("=?");
    int i=0;
    PreparedStatement statement=null;
    try{ 
      statement=conn.prepareStatement(cmd.toString());
      for(Map.Entry e: op.entrySet()){ String n=e.getKey().toString();
        if(n.equals(key)) continue;
        statement.setObject(++i,e.getValue());
      }
      statement.setObject(++i,op.get(key));
      return statement.executeUpdate();
    }finally{
      try{ statement.close(); }catch(Exception e){}
    }
  }

  public void insert(String table, Map<? extends Object, ? extends Object> op) throws SQLException{    
    StringBuilder cmd=new StringBuilder()
      .append("INSERT INTO ").append(table).append(" (");
    int count=0; String sep="";    
    for(Map.Entry e: op.entrySet()){ String n=e.getKey().toString();
      cmd.append(sep).append(n);
      sep=","; ++count;
    }
    cmd.append(") VALUES ("); sep="";
    for(int i=0; i<count; ++i){ 
      cmd.append(sep).append("?");
      sep=",";
    }
    cmd.append(")");    
    PreparedStatement statement=null;
    try{ 
      statement=conn.prepareStatement(cmd.toString());
      int i=0;
      for(Map.Entry e: op.entrySet()){ String n=e.getKey().toString();
        statement.setObject(++i,e.getValue());
      }
      statement.executeUpdate();
    }finally{
      try{ statement.close(); }catch(Exception e){}
    }
  }

  public void insertOrUpdate(String table, String key, Map op) throws SQLException{ 
    if( 0==update(table,key,op) ){
      insert(table,op);
    }
  }

  
  public static void main(String args[]) 
                             throws ClassNotFoundException, SQLException, IOException{
    DB db=new DB();
    
    db.delete("category", "name", "travel");
    
    /*db.insert("category", new ArrayMap(
        "name", "stamp",
        "text", "Blyegek"
    ));*/
    
    PrintWriter out=new PrintWriter("category.csv");
    CSVWriter.writeCSV( out,    
        db.select(" * FROM category")
    );
    out.close();

    for(Map cat: CSVReader.readCSV(LinkedHashMap.class,"category.csv") ){
      db.insertOrUpdate("category", "NAME", cat);
    }

    for(Map<String,Object> category: db.select("* FROM category")){
      System.out.println(category);
    }
    
    /*PrintWriter out=new PrintWriter(System.out);
    CSVWriter.writeCSV( out,    
        db.select(" * FROM product ORDER BY id")
    );
    out.close();*/

    /*XMLBuilder.writeXML(new File("product.xml"),
        db.select(" * FROM product ORDER BY id")
    );*/
    
    String cat="book"; 
    for(Map<String,Object> category 
                         :db.select("* FROM product WHERE category=${c}",
                                                                "c", cat)
    ){
      System.out.println(category);
    }
    
    db.close();
  }  
}
