package ij.util;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

public class IOUtil{  
  public enum ListFileMode { ALL, FILES_ONLY, DIRECTORIES_ONLY; }
  
  public static List<File> listFiles(List<File> list, File file, 
                                            ListFileMode mode, FileFilter filter){    
    if(file.isDirectory()){
      if( mode!=ListFileMode.FILES_ONLY){ list.add(file); } 
      File[] files=file.listFiles();
      if(files!=null){
        for(File f: files){
          listFiles(list,f,mode, filter); 
        }
      }
    }else{
      if( mode!=ListFileMode.DIRECTORIES_ONLY 
          &&(filter==null || filter.accept(file))){
        list.add(file);
      }
    }
    return list;
  }  
  public static List<File> listFiles(List<File> list, File file, 
                                                      FileFilter filter){ 
    return listFiles(list,file,ListFileMode.ALL,filter);
  }   
  public static List<File> listFiles(File file, ListFileMode mode, 
                                                      FileFilter filter){ 
    return listFiles(new ArrayList<File>(),file,mode,filter);
  }   
  public static List<File> listFiles(List<File> list, File file){ 
    return listFiles(list,file,null);
  }
  public static List<File> listFiles(File file, FileFilter filter){ 
    return listFiles(new ArrayList<File>(),file,filter);
  }
  public static List<File> listFiles(File file){ return listFiles(file,null);  }

  public static boolean delete(File file, FileFilter filter){    
    List<File> list=listFiles(file,filter); boolean success=true;
    for(int i=list.size(); --i>=0; ){
      //success&=list.get(i).delete();
      System.out.println("DELETE:"+list.get(i));
    }
    return success;
  }
  public static boolean delete(File file){    
    return delete(file,null);
  }

  public static boolean contentEquals(File fileA, File fileB)
                                          throws IOException{    
    InputStream inA=null, inB=null;
    try{
      inA=new FileInputStream(fileA);
      inB=new FileInputStream(fileB);
      int bA, bB;
      do{ 
        if( (bA=inA.read())!=(bB=inB.read()) ) return false;
      }while(bA!=-1); assert (bB==-1);
    }finally{
      try{ if(inA!=null) inA.close(); }catch(IOException e){ }
      try{ if(inB!=null) inB.close(); }catch(IOException e){ }
    }
    return true;
  }

  
  public static void main(String args[]) throws IOException{
    /*
    System.out.println(listFiles(new File(".")) );
    System.out.println( listFiles(new File(".").getAbsoluteFile()) );
    System.out.println( listFiles(new File("."), ListFileMode.FILES_ONLY, 
      new FileFilter(){
        public boolean accept(File f){
          return f.getName().endsWith(".java");      
        }
      }) 
    );
    
    delete(new File("."), 
      new FileFilter(){
        public boolean accept(File f){ 
          return f.getName().endsWith(".bak");
        }
      }
    );
    delete(new File("save"));  */  
    
    System.out.println(
        contentEquals(new File("readme.txt"),new File("readme.txt"))
    );
    System.out.println(
        contentEquals(new File("readme.txt"),new File("readme2.txt"))
    );
  }
}

