package ij.util;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class IOUtil{  
  public enum ListFileMode { ALL, FILES_ONLY, DIRECTORIES_ONLY; }
  
  public static List<File> listFiles(List<File> list, File file, 
                                            ListFileMode mode, FileFilter filter){    
    if(file.isDirectory()){
      if( mode!=ListFileMode.FILES_ONLY){ list.add(file); } 
      File[] files=file.listFiles();
      if(files!=null){
        for(File f: files){
          listFiles(list,f,mode, filter); 
        }
      }
    }else{
      if( mode!=ListFileMode.DIRECTORIES_ONLY 
          &&(filter==null || filter.accept(file))){
        list.add(file);
      }
    }
    return list;
  }  
  public static List<File> listFiles(List<File> list, File file, 
                                                      FileFilter filter){ 
    return listFiles(list,file,ListFileMode.ALL,filter);
  }   
  public static List<File> listFiles(File file, ListFileMode mode, 
                                                      FileFilter filter){ 
    return listFiles(new ArrayList<File>(),file,mode,filter);
  }   
  public static List<File> listFiles(List<File> list, File file){ 
    return listFiles(list,file,null);
  }
  public static List<File> listFiles(File file, FileFilter filter){ 
    return listFiles(new ArrayList<File>(),file,filter);
  }
  public static List<File> listFiles(File file){ return listFiles(file,null);  }

  public static boolean delete(File file, FileFilter filter){    
    List<File> list=listFiles(file,filter); boolean success=true;
    for(int i=list.size(); --i>=0; ){
      //success&=list.get(i).delete();
      System.out.println("DELETE:"+list.get(i));
    }
    return success;
  }
  public static boolean delete(File file){    
    return delete(file,null);
  }

  public static boolean contentEquals(File fileA, File fileB)
                                          throws IOException{    
    if(fileA.length()!=fileB.length()) return false;    
    InputStream inA=null, inB=null;
    try{
      inA=new BufferedInputStream(new FileInputStream(fileA));
      inB=new BufferedInputStream(new FileInputStream(fileB));
      int bA, bB;
      while( (bA=inA.read())==(bB=inB.read()) && bA!=-1 && bB!=-1){ }
      if(bA!=bB) return false;
    }finally{
      try{ if(inA!=null) inA.close(); }catch(IOException e){ }
      try{ if(inB!=null) inB.close(); }catch(IOException e){ }
    }
    return true;
  }

  public static void copyContent(File source, File target) throws IOException{
    if(source.isDirectory()){
      target.mkdir();
      for(File f: source.listFiles()){
        copyContent(f,new File(target,f.getName()));  
      }
    }else{
      InputStream in=null; OutputStream out=null;
      try{ 
        in=new BufferedInputStream(new FileInputStream(source));
        out=new BufferedOutputStream(new FileOutputStream(target));
        byte B[]=new byte[1024]; int length;
        while( (length=in.read(B))>0 ){ out.write(B,0,length); }         
      }finally{
        if(in!=null) try{ in.close(); }catch(IOException e){}
        if(out!=null) try{ out.close(); }catch(IOException e){}
      }
    }
  }
  public static void copy(File source, File target) throws IOException{   
    if(target.getParentFile()!=null) target.getParentFile().mkdirs();
    copyContent(source,target);
  }

  public static void zip(List<File> list, File zip) throws IOException{
    if(zip.getParent()!=null) zip. getParentFile().mkdirs();     
    ZipOutputStream out = null;
    try{   
      out = new ZipOutputStream(new FileOutputStream(zip));  
      InputStream in=null;
      for(File f: list){ if(!f.isFile()) continue;
        ZipEntry entry = new ZipEntry(f.toString());          
        entry.setTime(f.lastModified());
        out.putNextEntry(entry);
        try{                                                
          in=new BufferedInputStream(new FileInputStream(f));
          byte B[] = new byte[1024]; int count;
          while( (count = in.read(B))>0 ){ out.write(B, 0, count);}
        }finally{ 
          if(in!=null) try{ in.close(); }catch(IOException e){}
        }
      }
    }finally{
      if(out!=null) try{ out.close(); }catch(IOException e){}
    }
  }

  public static void zip(File source, FileFilter filter, File zip) 
                                                      throws IOException{    
    List<File> list=listFiles(source,ListFileMode.FILES_ONLY,filter);
    zip(list,zip);
  }
  public static void zip(File source, File zip) throws IOException{
    zip(source,null,zip);
  }
  
  public static void dump(File file) throws IOException{
    Reader in=null;
    try{ 
      in=new BufferedReader(new FileReader(file));
      int c;
      while( (c=in.read())!=-1 ){
        System.out.print((char)c);      
      }
    }finally{
      try{ in.close(); }catch(Exception e){}
    }
  }

  
  public static void main(String args[]) throws IOException{
    /*
    System.out.println(listFiles(new File(".")) );
    System.out.println( listFiles(new File(".").getAbsoluteFile()) );
    System.out.println( listFiles(new File("."), ListFileMode.FILES_ONLY, 
      new FileFilter(){
        public boolean accept(File f){
          return f.getName().endsWith(".java");      
        }
      }) 
    );
    
    delete(new File("."), 
      new FileFilter(){
        public boolean accept(File f){ 
          return f.getName().endsWith(".bak");
        }
      }
    );
    delete(new File("save"));  */  
    
    /*System.out.println(
        contentEquals(new File("readme.txt"),new File("readme.txt"))
    );
    System.out.println(
        contentEquals(new File("readme.txt"),new File("readme2.txt"))
    );*/
    
    // copy(new File("readme.txt"),new File("readme.new.txt"));
    
    //zip(new File("src"),new File("src.zip"));
    
    dump(new File("readme.txt"));
  }
}

