package ij.util;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;

public class IOUtil{  
  public enum ListFileMode { ALL, FILES_ONLY, DIRECTORIES_ONLY; }
  
  public static List<File> listFiles(List<File> list, File file, 
                                            ListFileMode mode, FileFilter filter){    
    if(file.isDirectory()){
      if( mode!=ListFileMode.FILES_ONLY){ list.add(file); } 
      File[] files=file.listFiles();
      if(files!=null){
        for(File f: files){
          listFiles(list,f,mode, filter); 
        }
      }
    }else{
      if( mode!=ListFileMode.DIRECTORIES_ONLY 
          &&(filter==null || filter.accept(file))){
        list.add(file);
      }
    }
    return list;
  }  
  public static List<File> listFiles(List<File> list, File file, 
                                                      FileFilter filter){ 
    return listFiles(list,file,ListFileMode.ALL,filter);
  }   
  public static List<File> listFiles(File file, ListFileMode mode, 
                                                      FileFilter filter){ 
    return listFiles(new ArrayList<File>(),file,mode,filter);
  }   

  public static List<File> listFiles(List<File> list, File file){ 
    return listFiles(list,file,null);
  }
  public static List<File> listFiles(File file, FileFilter filter){ 
    return listFiles(new ArrayList<File>(),file,filter);
  }
  public static List<File> listFiles(File file){ 
    return listFiles(file,null);
  }

  public static void main(String args[]){
    System.out.println(listFiles(new File(".")) );
    System.out.println( listFiles(new File(".").getAbsoluteFile()) );
    System.out.println( listFiles(new File("."), ListFileMode.FILES_ONLY, 
      new FileFilter(){
        public boolean accept(File f){
          return f.getName().endsWith(".java");      
        }
      }) 
    );
  }
}

