package ij.util;

import java.io.FileInputStream;
import java.io.IOException;

import javax.xml.parsers.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

public class XMLReader<T> extends DefaultHandler{
  public static boolean isFiller(CharSequence s){ 
    int lineCount=0;
    for(int i=0, l=s.length(); i<l; ++i){ char c=s.charAt(i);
      if(c=='\n' && ++lineCount>1) return false;
      if(!Character.isWhitespace(c)) return false;
    }
    return lineCount==1;
  }
  
  protected T result;
  public T getResult(){ return result; }

  public XMLReader(){ }
  public XMLReader(T result){ this.result=result; }  
  
  protected StringBuilder text=new StringBuilder();
  protected String textName;
  protected boolean hasText;
  protected StringBuilder path=new StringBuilder("/");

  public void start(CharSequence path, String name, Attributes attributes) throws SAXException{
    System.out.println('<'+name);
  }
  public void set(CharSequence path, String name, CharSequence value) throws SAXException{
    System.out.println(name+"="+value);
  }
  public void end(CharSequence path, String name) throws SAXException{
    System.out.println('>'+name);
  }  
  
  public void startElement(String uri, String localName, String qName,
                                    Attributes attributes) throws SAXException{
    if(hasText && textName!=null && !isFiller(text)) set(path,textName,text);
    textName=localName; text.setLength(0); hasText=false; 
    path.append(localName).append('/');      
    start(path,localName,attributes);
  }
  public void characters(char[] ch, int start, int length) throws SAXException{
    if(textName!=null){ hasText=true; text.append(ch,start,length); }
  }
  public void endElement(String uri, String localName, String qName) throws SAXException{
    if(hasText && textName!=null && !isFiller(text)) set(path,textName,text);
    end(path,localName);
    path.setLength(path.lastIndexOf("/",path.length()-2)+1);    
    textName=null;
  }
  
  protected SAXParser getSAXParser() throws SAXException, ParserConfigurationException{
    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setNamespaceAware(true); factory.setValidating(false);
    return factory.newSAXParser();
  }
  public T parse(InputSource inputSource) throws SAXException, ParserConfigurationException, IOException{
    getSAXParser().parse( inputSource, this );
    return getResult();
  }
  public T parse(String fname, String encoding) throws SAXException, ParserConfigurationException, IOException{
    InputSource inputSource =new InputSource(new FileInputStream(fname));
    if(encoding!=null) inputSource.setEncoding(encoding);
    return parse(inputSource);
  }
  public T parse(String fname) throws SAXException, ParserConfigurationException, IOException{
    return parse(fname,null);
  }
  
  public static void main(String args[]) throws SAXException, ParserConfigurationException, IOException{
    new XMLReader().parse("dvd.xml");
  }  
  
}
